#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time     : 
# @Author   : wgh


"""
文件说明：模型初始化
"""
import os
from FourthDimension.model.HuggingFaceBgeEmbeddings import HuggingFaceBgeEmbeddings

os.environ['HF_ENDPOINT'] = "https://hf-mirror.com"


def init_embeddings_model(model_name, model_kwargs, encode_kwargs):
    embeddings = HuggingFaceBgeEmbeddings(
        model_name=model_name,
        model_kwargs=model_kwargs,
        encode_kwargs=encode_kwargs
    )
    return embeddings



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time     : 
# @Author   : wgh
from FourthDimension.es.es_client import ElasticsearchClient
from FourthDimension.faiss_process.faiss_storage import embeddings_storage

es_client = ElasticsearchClient()


def upload_entrance(contexts):
    """
    存储入口
    :param contexts: 段落
    :return:
    """
    print('解析完成，文档上传中...')
    filter_context = es_upload(contexts)
    insert_file_name = faiss_upload(filter_context)
    print('已存储文档：{}'.format(insert_file_name))
    print('---------------------------------')


def es_upload(contexts):
    """
    es上传
    :param contexts: 段落
    :return:
    """
    filter_context = es_client.insert_data(contexts)
    return filter_context


def faiss_upload(contexts):
    """
    es上传
    :param contexts: 段落
    :return:
    """
    insert_file_name = embeddings_storage(contexts)
    return insert_file_name


def upload_test(contexts_path):
    pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time     : 
# @Author   : wgh


"""
文件说明：
"""
import os

from FourthDimension.config.config import config_setting, embedding_model
from FourthDimension.faiss_process.FAISS import FAISS

top_k = config_setting['recall_config']['top_k']

# 获取当前脚本文件的路径
current_path = os.path.abspath(__file__)

# 获取当前脚本文件所在的目录
current_dir = os.path.dirname(current_path)

index_file = current_dir + '/../cache/faiss_cache'


def faiss_search(question):
    top_k_contexts = []
    db = FAISS.load_local(index_file, embedding_model)
    docs = db.similarity_search(question, k=top_k)
    for i, doc in enumerate(docs):
        top_k_contexts.append(doc.page_content)
    return top_k_contexts



import os
import tempfile
from abc import ABC, abstractmethod
from typing import Iterator, List, Optional

import requests
from urllib.parse import urlparse

from FourthDimension.doc.document import Document
from FourthDimension.doc.spliter import TextSplitter, RecursiveCharacterTextSplitter


class BaseLoader(ABC):
    """Interface for Document Loader.

    Implementations should implement the lazy-loading method using generators
    to avoid loading all Documents into memory at once.

    The `load` method will remain as is for backwards compatibility, but its
    implementation should be just `list(self.lazy_load())`.
    """

    # Sub-classes should implement this method
    # as return list(self.lazy_load()).
    # This method returns a List which is materialized in memory.
    @abstractmethod
    def load(self) -> List[Document]:
        """Load data into Document objects."""

    def load_and_split(
        self, text_splitter: Optional[TextSplitter] = None
    ) -> List[Document]:
        """Load Documents and split into chunks. Chunks are returned as Documents.

        Args:
            text_splitter: TextSplitter instance to use for splitting documents.
              Defaults to RecursiveCharacterTextSplitter.

        Returns:
            List of Documents.
        """
        if text_splitter is None:
            _text_splitter: TextSplitter = RecursiveCharacterTextSplitter()
        else:
            _text_splitter = text_splitter
        docs = self.load()
        return _text_splitter.split_documents(docs)

    # Attention: This method will be upgraded into an abstractmethod once it's
    #            implemented in all the existing subclasses.
    def lazy_load(
        self,
    ) -> Iterator[Document]:
        """A lazy loader for Documents."""
        raise NotImplementedError(
            f"{self.__class__.__name__} does not implement lazy_load()"
        )


class Docx2txtLoader(BaseLoader, ABC):
    """Load `DOCX` file using `docx2txt` and chunks at character level.

    Defaults to check for local file, but if the file is a web path, it will download it
    to a temporary file, and use that, then clean up the temporary file after completion
    """

    def __init__(self, file_path: str):
        """Initialize with file path."""
        self.file_path = file_path
        if "~" in self.file_path:
            self.file_path = os.path.expanduser(self.file_path)

        # If the file is a web path, download it to a temporary file, and use that
        if not os.path.isfile(self.file_path) and self._is_valid_url(self.file_path):
            r = requests.get(self.file_path)

            if r.status_code != 200:
                raise ValueError(
                    "Check the url of your file; returned status code %s"
                    % r.status_code
                )

            self.web_path = self.file_path
            self.temp_file = tempfile.NamedTemporaryFile()
            self.temp_file.write(r.content)
            self.file_path = self.temp_file.name
        elif not os.path.isfile(self.file_path):
            raise ValueError("File path %s is not a valid file or url" % self.file_path)

    def __del__(self) -> None:
        if hasattr(self, "temp_file"):
            self.temp_file.close()

    def load(self) -> List[Document]:
        """Load given path as single page."""
        import docx2txt

        return [
            Document(
                page_content=docx2txt.process(self.file_path),
                metadata={"source": self.file_path},
            )
        ]

    @staticmethod
    def _is_valid_url(url: str) -> bool:
        """Check if the url is valid."""
        parsed = urlparse(url)
        return bool(parsed.netloc) and bool(parsed.scheme)


from __future__ import annotations
from abc import ABC, abstractmethod
from typing import Any, Dict, List, Literal, Optional, TypedDict, Union, cast, Sequence
import asyncio
from functools import partial

from FourthDimension.doc.pydantic import BaseModel, PrivateAttr, Field




class BaseSerialized(TypedDict):
    """Base class for serialized objects."""

    lc: int
    id: List[str]


class SerializedConstructor(BaseSerialized):
    """Serialized constructor."""

    type: Literal["constructor"]
    kwargs: Dict[str, Any]


class SerializedSecret(BaseSerialized):
    """Serialized secret."""

    type: Literal["secret"]


class SerializedNotImplemented(BaseSerialized):
    """Serialized not implemented."""

    type: Literal["not_implemented"]
    repr: Optional[str]


def try_neq_default(value: Any, key: str, model: BaseModel) -> bool:
    try:
        return model.__fields__[key].get_default() != value
    except Exception:
        return True

def _replace_secrets(
    root: Dict[Any, Any], secrets_map: Dict[str, str]
) -> Dict[Any, Any]:
    result = root.copy()
    for path, secret_id in secrets_map.items():
        [*parts, last] = path.split(".")
        current = result
        for part in parts:
            if part not in current:
                break
            current[part] = current[part].copy()
            current = current[part]
        if last in current:
            current[last] = {
                "lc": 1,
                "type": "secret",
                "id": [secret_id],
            }
    return result


def to_json_not_implemented(obj: object) -> SerializedNotImplemented:
    """Serialize a "not implemented" object.

    Args:
        obj: object to serialize

    Returns:
        SerializedNotImplemented
    """
    _id: List[str] = []
    try:
        if hasattr(obj, "__name__"):
            _id = [*obj.__module__.split("."), obj.__name__]
        elif hasattr(obj, "__class__"):
            _id = [*obj.__class__.__module__.split("."), obj.__class__.__name__]
    except Exception:
        pass

    result: SerializedNotImplemented = {
        "lc": 1,
        "type": "not_implemented",
        "id": _id,
        "repr": None,
    }
    try:
        result["repr"] = repr(obj)
    except Exception:
        pass
    return result

class Serializable(BaseModel, ABC):
    """Serializable base class."""

    @classmethod
    def is_lc_serializable(cls) -> bool:
        """Is this class serializable?"""
        return False

    @classmethod
    def get_lc_namespace(cls) -> List[str]:
        """Get the namespace of the langchain object.

        For example, if the class is `langchain.llms.openai.OpenAI`, then the
        namespace is ["langchain", "llms", "openai"]
        """
        return cls.__module__.split(".")

    @property
    def lc_secrets(self) -> Dict[str, str]:
        """A map of constructor argument names to secret ids.

        For example,
            {"openai_api_key": "OPENAI_API_KEY"}
        """
        return dict()

    @property
    def lc_attributes(self) -> Dict:
        """List of attribute names that should be included in the serialized kwargs.

        These attributes must be accepted by the constructor.
        """
        return {}

    @classmethod
    def lc_id(cls) -> List[str]:
        """A unique identifier for this class for serialization purposes.

        The unique identifier is a list of strings that describes the path
        to the object.
        """
        return [*cls.get_lc_namespace(), cls.__name__]

    class Config:
        extra = "ignore"

    def __repr_args__(self) -> Any:
        return [
            (k, v)
            for k, v in super().__repr_args__()
            if (k not in self.__fields__ or try_neq_default(v, k, self))
        ]

    _lc_kwargs = PrivateAttr(default_factory=dict)

    def __init__(self, **kwargs: Any) -> None:
        super().__init__(**kwargs)
        self._lc_kwargs = kwargs

    def to_json(self) -> Union[SerializedConstructor, SerializedNotImplemented]:
        if not self.is_lc_serializable():
            return self.to_json_not_implemented()

        secrets = dict()
        # Get latest values for kwargs if there is an attribute with same name
        lc_kwargs = {
            k: getattr(self, k, v)
            for k, v in self._lc_kwargs.items()
            if not (self.__exclude_fields__ or {}).get(k, False)  # type: ignore
        }

        # Merge the lc_secrets and lc_attributes from every class in the MRO
        for cls in [None, *self.__class__.mro()]:
            # Once we get to Serializable, we're done
            if cls is Serializable:
                break

            if cls:
                deprecated_attributes = [
                    "lc_namespace",
                    "lc_serializable",
                ]

                for attr in deprecated_attributes:
                    if hasattr(cls, attr):
                        raise ValueError(
                            f"Class {self.__class__} has a deprecated "
                            f"attribute {attr}. Please use the corresponding "
                            f"classmethod instead."
                        )

            # Get a reference to self bound to each class in the MRO
            this = cast(Serializable, self if cls is None else super(cls, self))

            secrets.update(this.lc_secrets)
            lc_kwargs.update(this.lc_attributes)

        # include all secrets, even if not specified in kwargs
        # as these secrets may be passed as an environment variable instead
        for key in secrets.keys():
            secret_value = getattr(self, key, None) or lc_kwargs.get(key)
            if secret_value is not None:
                lc_kwargs.update({key: secret_value})

        return {
            "lc": 1,
            "type": "constructor",
            "id": self.lc_id(),
            "kwargs": lc_kwargs
            if not secrets
            else _replace_secrets(lc_kwargs, secrets),
        }

    def to_json_not_implemented(self) -> SerializedNotImplemented:
        return to_json_not_implemented(self)


class Document(Serializable):
    """Class for storing a piece of text and associated metadata."""

    page_content: str
    """String text."""
    metadata: dict = Field(default_factory=dict)
    """Arbitrary metadata about the page content (e.g., source, relationships to other
        documents, etc.).
    """
    type: Literal["Document"] = "Document"

    @classmethod
    def is_lc_serializable(cls) -> bool:
        """Return whether this class is serializable."""
        return True


class BaseDocumentTransformer(ABC):
    """Abstract base class for document transformation systems.

    A document transformation system takes a sequence of Documents and returns a
    sequence of transformed Documents.

    Example:
        .. code-block:: python

            class EmbeddingsRedundantFilter(BaseDocumentTransformer, BaseModel):
                embeddings: Embeddings
                similarity_fn: Callable = cosine_similarity
                similarity_threshold: float = 0.95

                class Config:
                    arbitrary_types_allowed = True

                def transform_documents(
                    self, documents: Sequence[Document], **kwargs: Any
                ) -> Sequence[Document]:
                    stateful_documents = get_stateful_documents(documents)
                    embedded_documents = _get_embeddings_from_stateful_docs(
                        self.embeddings, stateful_documents
                    )
                    included_idxs = _filter_similar_embeddings(
                        embedded_documents, self.similarity_fn, self.similarity_threshold
                    )
                    return [stateful_documents[i] for i in sorted(included_idxs)]

                async def atransform_documents(
                    self, documents: Sequence[Document], **kwargs: Any
                ) -> Sequence[Document]:
                    raise NotImplementedError

    """  # noqa: E501

    @abstractmethod
    def transform_documents(
        self, documents: Sequence[Document], **kwargs: Any
    ) -> Sequence[Document]:
        """Transform a list of documents.

        Args:
            documents: A sequence of Documents to be transformed.

        Returns:
            A list of transformed Documents.
        """

    async def atransform_documents(
        self, documents: Sequence[Document], **kwargs: Any
    ) -> Sequence[Document]:
        """Asynchronously transform a list of documents.

        Args:
            documents: A sequence of Documents to be transformed.

        Returns:
            A list of transformed Documents.
        """
        return await asyncio.get_running_loop().run_in_executor(
            None, partial(self.transform_documents, **kwargs), documents
        )


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time     : 
# @Author   : wgh


"""
文件说明：
"""
import re

import jieba
from fuzzywuzzy import fuzz

from FourthDimension.config.config import patterns, patternsLen


class DocQAAnswerEntity:
    def init(self, content, question, fuzz_score, words_ratio):
        self.content = content
        self.question = question
        self.fuzz_score = fuzz_score
        self.words_ratio = words_ratio


def init_question_template(file_path):
    question_template = []
    with open(file_path, 'r', encoding='utf-8') as file:
        for line in file:
            # 去除行尾的换行符
            line = line.strip()
            question_template.append(line)
    for i, line in enumerate(question_template):
        patterns.append(re.compile(line))


def merge(intervals):
    intervals.sort(key=lambda x: x[0])
    res = []
    for i in range(len(intervals)):
        l, r = intervals[i][0], intervals[i][1]
        if len(res) == 0 or res[-1][1] < l:
            res.append([l, r])
        else:
            res[-1][1] = max(res[-1][1], r)
    return res


def question_analysis(question):
    find = False
    query = question.replace("，", "")
    matchIndex = []
    for i in range(patternsLen):
        regex = patterns[i]
        matcher = re.search(regex, query)
        if matcher is not None:
            find = True
            for match in re.finditer(regex, query):
                start = match.start()
                end = match.end()
                matchIndex.append([start, end])
    if find:
        mergeIndex = merge(matchIndex)
        res = []
        startI = 0
        for i in range(len(mergeIndex)):
            if startI != mergeIndex[i][0]:
                res.append(query[startI:mergeIndex[i][0]])
            startI = mergeIndex[i][1]
        if mergeIndex[-1][1] < len(query):
            res.append(query[mergeIndex[-1][1]:])
        return ''.join(res)
    return query


def getDetailResult(datas, question, question_anal):
    question_analysisCut = jieba.lcut(question_anal)
    for scoreContent in datas:
        content = scoreContent.content
        content = content.lower().replace(" ", "")
        cutContent = cut_sentence_short(content)
        max_score = -1
        max_len = -1
        for con in cutContent:
            score = fuzzSimilarity(question, con)
            length = longestLen(question_anal, con)
            max_score = max(max_score, score)
            max_len = max(max_len, length)
        cnt = 0
        exist_words = 0
        for word in question_analysisCut:
            count = filter(content, word)
            if count != 0:
                exist_words += 1
            cnt += count
        quesSize = len(question_analysisCut)
        if quesSize == 0:
            scoreContent.words_ratio = 1.0
        else:
            scoreContent.words_ratio = exist_words / quesSize * max_len
        scoreContent.fuzz_score = max_score
    return es_sorted(datas)


def fuzzSimilarity(question, content):
    similarityScore = fuzz.token_sort_ratio(question, content)
    return similarityScore


def cut_sentence_short(content):
    regex = "[。！!?？;；]"
    cons = re.split(regex, content)
    return cons


def zeros(context_char_idxs):
    for i in range(len(context_char_idxs)):
        for j in range(len(context_char_idxs[i])):
            context_char_idxs[i][j] = 0
    return context_char_idxs


def longestLen(question_analysis, content):
    q_len = len(question_analysis)
    c_len = len(content)
    dp = [[0] * (c_len + 1) for _ in range(q_len + 1)]
    dp = zeros(dp)
    ques_char = list(question_analysis)
    con_char = list(content)
    for i in range(1, q_len + 1):
        for j in range(1, c_len + 1):
            if ques_char[i - 1] == con_char[j - 1]:
                dp[i][j] = dp[i - 1][j - 1] + 1
            else:
                dp[i][j] = max(dp[i - 1][j], dp[i][j - 1])
    return dp[q_len][c_len]


def filter(string, sub_str):
    index = 0
    count = 0
    while True:
        index = string.find(sub_str, index)
        if index == -1:
            break
        index += 1
        count += 1
    return count


def es_sorted(datas):
    sorted_compare(datas)
    return datas


def sorted_compare(datas):
    datas.sort(key=lambda o: (o.words_ratio, o.fuzz_score))
    datas.reverse()
    return datas



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time     : 
# @Author   : wgh
import openai

from FourthDimension.config.config import config_setting

answer_generation_model = config_setting['answer_generation_model']
openai.api_key = config_setting['openai']['api_key']
openai.api_base = config_setting['openai']['url']


def answer_generate(question, top_k_contexts):
    top_k_contexts_str = ""
    for i, d in enumerate(top_k_contexts):
        top_k_contexts_str += f"{i + 1}：{d}\n"
    prompt = "你的背景知识:{}；对话要求：1. 背景知识是最新的实时的信息，使用背景知识回答问题。" \
             "2. 优先使用背景知识的内容回答我的问题，答案应与背景知识严格一致。" \
             "3. 背景知识无法回答我的问题时，可以忽略背景知识，根据你的知识来自由回答。" \
             "4. 使用对话的风格，自然的回答问题。我的问题是:{}".format(top_k_contexts_str, question)
    # prompt = "{}以上是所有段落并以序号进行了标注，你现在作为基于以上所有段落的的阅读理解模型，请阅读以上所有段落并选择一个最合理的从中提取问题：{}" \
    #          "的答案并输出：<answer>，如果无答案则输出无答案".format(top_k_contexts_str, question)

    chat_completion = openai.ChatCompletion.create(
        model=answer_generation_model, messages=[{"role": "user", "content": prompt}]
    )
    answer = chat_completion.choices[0].message.content
    return answer



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time     : 
# @Author   : wgh

from elasticsearch import Elasticsearch

from FourthDimension.config.config import config_setting
from FourthDimension.es.es_sort import (
    question_analysis,
    getDetailResult
)

"""
文件说明：
"""
# Elasticsearch连接信息
host = config_setting['elasticsearch_setting']['host']
port = config_setting['elasticsearch_setting']['port']
username = config_setting['elasticsearch_setting']['username']
password = config_setting['elasticsearch_setting']['password']
index_name = config_setting['elasticsearch_setting']['index_name']
analyzer = config_setting['elasticsearch_setting']['analyzer']


class DocQAAnswerEntity:
    def init(self, content, question, fuzz_score, words_ratio):
        self.content = content
        self.question = question
        self.fuzz_score = fuzz_score
        self.words_ratio = words_ratio


class ElasticsearchClient:
    def __init__(self, host=host, port=port, username=username, password=password):
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.client = Elasticsearch(
            [self.host],
            http_auth=(self.username, self.password),
            port=self.port,
            use_ssl=False,
            verify_certs=False,
            timeout=30,
            max_retries=3,
            retry_on_timeout=True
        )

    def create_index(self):
        index_exists = self.client.indices.exists(index=index_name)
        if index_exists:
            # print(f"索引 '{index_name}' 已存在")
            pass
        else:
            # 创建索引
            settings = {
                "settings": {
                    "number_of_shards": 1,
                    "number_of_replicas": 0
                },
                "index": {
                    "refresh_interval": "20s"
                },
                "mappings": {
                    "_doc": {
                        "dynamic": "strict",
                        "properties": {
                            "file_name": {
                                "type": "keyword"
                            },
                            "type": {
                                "type": "keyword"
                            },
                            "text": {
                                "type": "text",
                                "analyzer": analyzer,
                                "search_analyzer": analyzer
                            }
                        }
                    }
                }
            }
            self.client.indices.create(index=index_name, ignore=400, body=settings)
            print(f"索引 '{index_name}' 创建成功")

    def insert_data(self, all_context):
        file_names = self.search_filename()
        for i, d in enumerate(all_context):
            filename = d.metadata['source']
            if filename not in file_names:
                context = d.page_content
                self.client.index(index=index_name, doc_type='_doc', body={
                    'file_name': filename,
                    'type': 'part',
                    'text': context})
        return all_context

    def search_filename(self):
        # 构建查询语句
        query = {
            "size": 10000,
            "_source": ["file_name"],
            "query": {
                "match_all": {}
            }
        }
        # 执行查询
        result = self.client.search(index=index_name, body=query)

        # 从查询结果中提取文件名
        file_names = [hit["_source"]["file_name"] for hit in result["hits"]["hits"]]

        # 去重文件名列表
        unique_file_names = list(set(file_names))
        return unique_file_names

    def clean_data(self):
        """
        清除数据
        """
        self.client.delete_by_query(index=index_name, body={"query": {"match_all": {}}})

    def es_search(self, question):
        question_anal = question_analysis(question)
        searchHits = []
        try:
            # es查询
            sourceBuilder = {
                "query": {
                    "bool": {
                        "must": [
                            {
                                "bool": {
                                    "should": [
                                        {
                                            "match": {
                                                "text": {
                                                    "query": question,
                                                    "boost": 1,
                                                    "analyzer": analyzer
                                                }
                                            }
                                        },
                                        {
                                            "match": {
                                                "text": {
                                                    "query": question_anal,
                                                    "boost": 3,
                                                    "analyzer": analyzer
                                                }
                                            }
                                        }
                                    ]
                                }
                            },
                            {
                                "term": {
                                    "type": "part"
                                }
                            }
                        ],
                        "filter": [
                            {
                                "term": {
                                    "type": "part"
                                }
                            }
                        ]
                    }
                },
                "size": 10
            }
            response = self.client.search(index=index_name, body=sourceBuilder)
            hits = response["hits"]["hits"]
            searchHits = hits
        except Exception as e:
            print(e)

        esResponses = []

        # 获取es查询命中结果
        for hit in searchHits:
            esResponse = DocQAAnswerEntity()
            hitString = hit["_source"]
            content = hitString['text']
            esResponse.question = question
            esResponse.content = content
            esResponses.append(esResponse)
        sorted_es_response = getDetailResult(esResponses, question, question_anal)
        top_k_content = []
        for i, d in enumerate(sorted_es_response):
            top_k_content.append(d.content)
        return top_k_content



import os
import tempfile
from typing import Any, Optional
from uuid import UUID

from fastapi import UploadFile
from FourthDimension.doc.spliter import RecursiveCharacterTextSplitter
from pydantic import BaseModel
from FourthDimension.utils.file import compute_sha1_from_file
from FourthDimension.config.config import config_setting

chunk_size = config_setting['para_config']['chunk_size']
chunk_overlap = config_setting['para_config']['overlap']


class File(BaseModel):
    id: Optional[UUID] = None
    file: Optional[UploadFile]
    file_name: Optional[str] = ""
    file_size: Optional[int] = None
    file_sha1: Optional[str] = ""
    vectors_ids: Optional[list] = []
    file_extension: Optional[str] = ""
    content: Optional[Any] = None
    chunk_size: int = chunk_size
    chunk_overlap: int = chunk_overlap
    documents: Optional[Any] = None

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        if self.file:
            self.file_name = self.file.filename
            self.file_size = self.file.size  # pyright: ignore reportPrivateUsage=none
            self.file_extension = os.path.splitext(
                self.file.filename  # pyright: ignore reportPrivateUsage=none
            )[-1].lower()

    async def compute_file_sha1(self):
        """
        Compute the sha1 of the file using a temporary file
        """
        with tempfile.NamedTemporaryFile(
                delete=False,
                suffix=self.file.filename,  # pyright: ignore reportPrivateUsage=none
        ) as tmp_file:
            await self.file.seek(0)  # pyright: ignore reportPrivateUsage=none
            self.content = (
                await self.file.read()  # pyright: ignore reportPrivateUsage=none
            )
            tmp_file.write(self.content)
            tmp_file.flush()
            self.file_sha1 = compute_sha1_from_file(tmp_file.name)

        os.remove(tmp_file.name)

    def compute_documents(self, loader_class):
        """
        Compute the documents from the file

        Args:
            loader_class (class): The class of the loader to use to load the file
        """

        documents = []
        with tempfile.NamedTemporaryFile(
                delete=False,
                suffix=self.file.filename,  # pyright: ignore reportPrivateUsage=none
        ) as tmp_file:
            tmp_file.write(self.content)  # pyright: ignore reportPrivateUsage=none
            tmp_file.flush()
            loader = loader_class(tmp_file.name)
            documents = loader.load()

        os.remove(tmp_file.name)

        text_splitter = RecursiveCharacterTextSplitter.from_tiktoken_encoder(
            chunk_size=self.chunk_size, chunk_overlap=self.chunk_overlap
        )

        self.documents = text_splitter.split_documents(documents)



from importlib import metadata

try:
    from pydantic.v1 import *  # noqa: F403
except ImportError:
    from pydantic import *  # noqa: F403

try:
    _PYDANTIC_MAJOR_VERSION: int = int(metadata.version("pydantic").split(".")[0])
except metadata.PackageNotFoundError:
    _PYDANTIC_MAJOR_VERSION = 0


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time     : 
# @Author   : wgh


"""
文件说明：
"""
from FourthDimension.model.chatGPT import answer_generate


def generate_answers(question, data):
    """
    答案生成
    :param question: 问题
    :param data: 召回结果
    :return:
    """
    if len(data) > 0:
        print('检索已召回，答案生成中...')
        answer = answer_generate(question, data)
        print('答案：{}'.format(answer))
        print('---------------------------------')
        return answer
    else:
        print('无检索结果，请确认是否上传文档')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time     : 
# @Author   : wgh


"""
文件说明：
"""
from FourthDimension.utils.zutil import get_simi_score, chinese_segment
from FourthDimension.config.config import config_setting

top_k = config_setting['recall_config']['top_k']


def rerank(question, paras):
    para_scores = []
    for i, para in enumerate(paras):
        sents = chinese_segment(para)
        sents_score = []
        for context in sents:
            D = get_simi_score(question, context)
            sents_score.append({
                'sentence': context,
                'score': str(D[0][0])
            })
        result = [{k: float(v) if k == "score" else v for k, v in d.items()} for d in sents_score]
        sorted_dict_list = sorted(result, key=lambda x: x['score'])
        if len(sents) == 1:
            score = sum(d["score"] for d in sorted_dict_list) / len(sents)
        elif len(sents) == 2:
            score = sorted_dict_list[0]["score"] * 0.7 + sorted_dict_list[1]["score"]
        else:
            score = sorted_dict_list[0]["score"] * 0.5 + sorted_dict_list[1]["score"] * 0.3 + sorted_dict_list[2][
                "score"] * 0.2
        para_scores.append([i, score])
    sorted_para = sorted(para_scores, key=lambda x: x[1])
    paras_rerank_index = []
    for i in range(len(sorted_para)):
        paras_rerank_index.append(sorted_para[i][0])
    rerank_contexts = []
    for i, d in enumerate(paras_rerank_index):
        rerank_contexts.append(paras[d])
    return rerank_contexts



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time     : 
# @Author   : wgh


"""
文件说明：
"""
from pydantic import Field, Extra
from pydantic.main import BaseModel

from FourthDimension.schema.embeddings import Embeddings
from typing import Any, Dict, List, Optional

DEFAULT_BGE_MODEL = "BAAI/bge-large-en"
DEFAULT_QUERY_BGE_INSTRUCTION_EN = (
    "Represent this question for searching relevant passages: "
)
DEFAULT_QUERY_BGE_INSTRUCTION_ZH = "为这个句子生成表示以用于检索相关文章："


class HuggingFaceBgeEmbeddings(BaseModel, Embeddings):
    client: Any  #: :meta private:
    model_name: str = DEFAULT_BGE_MODEL
    """Model name to use."""
    cache_folder: Optional[str] = None
    """Path to store models.
    Can be also set by SENTENCE_TRANSFORMERS_HOME environment variable."""
    model_kwargs: Dict[str, Any] = Field(default_factory=dict)
    """Key word arguments to pass to the model."""
    encode_kwargs: Dict[str, Any] = Field(default_factory=dict)
    """Key word arguments to pass when calling the `encode` method of the model."""
    query_instruction: str = DEFAULT_QUERY_BGE_INSTRUCTION_EN
    """Instruction to use for embedding query."""

    def __init__(self, **kwargs: Any):
        """Initialize the sentence_transformer."""
        super().__init__(**kwargs)
        try:
            import sentence_transformers

        except ImportError as exc:
            raise ImportError(
                "Could not import sentence_transformers python package. "
                "Please install it with `pip install sentence_transformers`."
            ) from exc

        self.client = sentence_transformers.SentenceTransformer(
            self.model_name, cache_folder=self.cache_folder, **self.model_kwargs
        )
        if "-zh" in self.model_name:
            self.query_instruction = DEFAULT_QUERY_BGE_INSTRUCTION_ZH

    class Config:
        """Configuration for this pydantic object."""

        extra = Extra.forbid

    def embed_documents(self, texts: List[str]) -> List[List[float]]:
        """Compute doc embeddings using a HuggingFace transformer model.

        Args:
            texts: The list of texts to embed.

        Returns:
            List of embeddings, one for each text.
        """
        texts = [t.replace("\n", " ") for t in texts]
        embeddings = self.client.encode(texts, **self.encode_kwargs)
        return embeddings.tolist()

    def embed_query(self, text: str) -> List[float]:
        """Compute query embeddings using a HuggingFace transformer model.

        Args:
            text: The text to embed.

        Returns:
            Embeddings for the text.
        """
        text = text.replace("\n", " ")
        embedding = self.client.encode(
            self.query_instruction + text, **self.encode_kwargs
        )
        return embedding.tolist()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time     : 
# @Author   : wgh
from FourthDimension.config.config import config_setting
from FourthDimension.es.es_client import ElasticsearchClient
from FourthDimension.faiss_process.faiss_index import faiss_search
from FourthDimension.utils.mix_sort import rerank

word_storage = config_setting['word_storage']
embedding_storage = config_setting['embedding_storage']
search_select = config_setting['search_select']
index_name = config_setting['elasticsearch_setting']['index_name']

elasticsearch = 'elasticsearch'
faiss = 'faiss'
elasticsearch_faiss = 'default'

es_client = ElasticsearchClient()


def query_entrance(question):
    """
    查询入口
    :param question: 问题
    :return:
    """
    print('开始检索问题：{}'.format(question))
    top_k_contexts = []
    if search_select == elasticsearch:
        top_k_contexts = es_query(question)
    elif search_select == faiss:
        top_k_contexts = faiss_query(question)
    elif search_select == elasticsearch_faiss:
        top_k_contexts = es_faiss_query(question)
    else:
        print(f"参数search_select无法匹配，请检查参数：f{search_select}")
    return top_k_contexts


def es_query(question):
    """
    es检索增强生成
    :param question: 问题
    :return:
    """
    top_k_contexts = es_client.es_search(question)
    return top_k_contexts


def faiss_query(question):
    """
    faiss检索
    :param question: 问题
    :return:
    """
    top_k_contexts = faiss_search(question)
    return top_k_contexts


def es_faiss_query(question):
    """
    es+faiss重排查询
    :param question: 问题
    :return:
    """
    es_top_k_contexts = es_query(question)
    faiss_top_k_contexts = faiss_query(question)
    merged_top_k = list(set(es_top_k_contexts + faiss_top_k_contexts))
    rerank_result = rerank(question, merged_top_k)
    return rerank_result



import io
import os.path
import re

from fastapi import UploadFile
from FourthDimension.doc.loader import Docx2txtLoader

from FourthDimension.parser.files import File
from .common import process_file


def replace_multiple_newlines(text):
    pattern = r'\n{1,}'  # 匹配连续一个以上的换行符
    replacement = '\n'  # 替换为一个换行符
    updated_text = re.sub(pattern, replacement, text)
    return updated_text


def process_docx(file: File):
    document = process_file(
        file=file,
        loader_class=Docx2txtLoader
    )
    for i, d in enumerate(document):
        document[i].page_content = replace_multiple_newlines(document[i].page_content)
        document[i].metadata['source'] = os.path.basename(document[i].metadata['source'])
    return document


def parse_docx(file_path, file_name):
    print('正在解析文档：' + file_name)

    with open(file_path, "rb") as f:
        file_content = f.read()

    # Create a file-like object in memory using BytesIO
    file_object = io.BytesIO(file_content)
    upload_file = UploadFile(
        file=file_object, filename=file_name, size=len(file_content)
    )
    file_instance = File(file=upload_file)
    file_instance.content = file_content

    document = process_docx(file_instance)

    for i, doc in enumerate(document):
        document[i].metadata['source'] = file_name
    return document



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time     : 
# @Author   : wgh


"""
文件说明：
"""
import os
import shutil

from FourthDimension.config.config import embedding_model
from FourthDimension.faiss_process.FAISS import FAISS

# 获取当前脚本文件的路径
current_path = os.path.abspath(__file__)

# 获取当前脚本文件所在的目录
current_dir = os.path.dirname(current_path)

index_file = current_dir + '/../cache/faiss_cache'


def clean_faiss():
    if os.path.exists(index_file):
        shutil.rmtree(index_file)


def embeddings_storage(contexts):
    insert_data = []
    insert_file_name = []
    exist_file_name = []
    if os.path.exists(index_file):
        db = FAISS.load_local(index_file, embedding_model)
        db_data = db.docstore.__dict__
        for i, doc in enumerate(db_data['_dict'].items()):
            file_name = doc[1].metadata['source']
            exist_file_name.append(file_name)
        exist_file_name = list(set(exist_file_name))
        for i, doc in enumerate(contexts):
            file_name = doc.metadata['source']
            if file_name not in exist_file_name:
                insert_data.append(doc)
                insert_file_name.append(file_name)
        if len(insert_data) > 0:
            db.add_documents(insert_data)
            db.save_local(index_file)
    else:
        for i, doc in enumerate(contexts):
            file_name = doc.metadata['source']
            insert_file_name.append(file_name)
        db = FAISS.from_documents(contexts, embedding_model)
        db.save_local(index_file)
    insert_file_name = list(set(insert_file_name))
    return insert_file_name



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time     : 
# @Author   : wgh
from FourthDimension.es.es_client import ElasticsearchClient
from FourthDimension.faiss_process.faiss_storage import clean_faiss

es_client = ElasticsearchClient()


def clean_entrance():
    """
    清除数据入口
    """
    print('开始清除数据...')
    es_clean()
    print('数据清除中...')
    faiss_clean()
    print('数据已清除')
    print('---------------------------------')


def es_clean():
    """
    清除es中数据
    """
    es_client.clean_data()


def faiss_clean():
    """
    清空faiss数据
    :return:
    """
    clean_faiss()



import hashlib

from fastapi import UploadFile


def convert_bytes(bytes, precision=2):
    """Converts bytes into a human-friendly format."""
    abbreviations = ["B", "KB", "MB"]
    if bytes <= 0:
        return "0 B"
    size = bytes
    index = 0
    while size >= 1024 and index < len(abbreviations) - 1:
        size /= 1024
        index += 1
    return f"{size:.{precision}f} {abbreviations[index]}"


def get_file_size(file: UploadFile):
    # move the cursor to the end of the file
    file.file._file.seek(0, 2)  # pyright: ignore reportPrivateUsage=none
    file_size = (
        file.file._file.tell()  # pyright: ignore reportPrivateUsage=none
    )  # Getting the size of the file
    # move the cursor back to the beginning of the file
    file.file.seek(0)

    return file_size


def compute_sha1_from_file(file_path):
    with open(file_path, "rb") as file:
        bytes = file.read()
        readable_hash = compute_sha1_from_content(bytes)
    return readable_hash


def compute_sha1_from_content(content):
    readable_hash = hashlib.sha1(content).hexdigest()
    return readable_hash



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time     : 
# @Author   : wgh
import json
import logging
import os
import re

from FourthDimension.model.model_init import init_embeddings_model

"""
文件说明：项目初始化
"""
patterns = []
question_template = []
config_setting = {}
patternsLen = 0
embedding_model = None

# 获取当前脚本文件的路径
current_path = os.path.abspath(__file__)

# 获取当前脚本文件所在的目录
current_dir = os.path.dirname(current_path)

# 获取项目根目录的路径
root_dir = os.path.dirname(current_dir)

# 读取配置文件
with open('config.json') as f:
    config_setting = dict(json.load(f))

config_setting.setdefault('elasticsearch_setting', {})
elasticsearch_setting = config_setting['elasticsearch_setting']
elasticsearch_setting.update({
    'host': 'host',
    'port': 9200,
    'username': '',
    'password': '',
    'index_name': 'default_index',
    'analyzer': 'standard'
})

# 读取问句模板
with open(current_dir + '/question_regex.txt', 'r', encoding='utf-8') as file:
    for line in file:
        line = line.strip()
        question_template.append(line)

for i, line in enumerate(question_template):
    patterns.append(re.compile(line))

patternsLen = len(patterns)
logging.warning('模型初始化...')
embedding_model = init_embeddings_model(model_name=config_setting['embedding_model'],
                                        model_kwargs={'device': 'cuda'},
                                        encode_kwargs={'normalize_embeddings': True})



from setuptools import setup, find_packages

VERSION = '1.0.5b1'
DESCRIPTION = 'FourthDimension（第四维度）由华中科技大学人工智能与嵌入式实验室联合言图科技研发，是一款基于大语言模型的智能检索增强生成（RAG）系统，提供私域知识库、文档问答等多种服务。此外，FourthDimension提供便捷的本地部署方法，方便用户在本地环境中搭建属于自己的应用平台。'

setup(
    name="FourthDimension",
    version=VERSION,
    author="yantu-tech",
    author_email="wu466687121@qq.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=open('README.md', encoding="UTF8").read(),
    package_dir={"": "src"},
    packages=find_packages("src"),
    include_package_data=True,
    package_data={'FourthDimension': ['config/question_regex.txt', 'cache/faiss_cache/temp.index']},
    install_requires=[],
    keywords=["python", "yantu"],
    license="MIT",
    url="https://gitee.com/hustai/FourthDimension",
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3"
    ]
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time     : 
# @Author   : wgh
"""Schemas for the LangSmith API."""
from __future__ import annotations

from datetime import datetime, timedelta
from enum import Enum
from typing import Any, Dict, List, Optional, Protocol, Union, runtime_checkable
from uuid import UUID

try:
    from pydantic.v1 import (  # type: ignore[import]
        BaseModel,
        Field,
        PrivateAttr,
        StrictBool,
        StrictFloat,
        StrictInt,
    )
except ImportError:
    from pydantic import (
        BaseModel,
        Field,
        PrivateAttr,
        StrictBool,
        StrictFloat,
        StrictInt,
    )

from typing_extensions import Literal

SCORE_TYPE = Union[StrictBool, StrictInt, StrictFloat, None]
VALUE_TYPE = Union[Dict, StrictBool, StrictInt, StrictFloat, str, None]


class ExampleBase(BaseModel):
    """Example base model."""

    dataset_id: UUID
    inputs: Dict[str, Any]
    outputs: Optional[Dict[str, Any]] = Field(default=None)

    class Config:
        frozen = True


class ExampleCreate(ExampleBase):
    """Example create model."""

    id: Optional[UUID]
    created_at: datetime = Field(default_factory=datetime.utcnow)


class Example(ExampleBase):
    """Example model."""

    id: UUID
    created_at: datetime
    modified_at: Optional[datetime] = Field(default=None)
    runs: List[Run] = Field(default_factory=list)


class ExampleUpdate(BaseModel):
    """Update class for Example."""

    dataset_id: Optional[UUID] = None
    inputs: Optional[Dict[str, Any]] = None
    outputs: Optional[Dict[str, Any]] = None

    class Config:
        frozen = True


class DataType(str, Enum):
    """Enum for dataset data types."""

    kv = "kv"
    llm = "llm"
    chat = "chat"


class DatasetBase(BaseModel):
    """Dataset base model."""

    name: str
    description: Optional[str] = None
    data_type: Optional[DataType] = None

    class Config:
        frozen = True


class DatasetCreate(DatasetBase):
    """Dataset create model."""

    id: Optional[UUID] = None
    created_at: datetime = Field(default_factory=datetime.utcnow)


class Dataset(DatasetBase):
    """Dataset ORM model."""

    id: UUID
    created_at: datetime
    modified_at: Optional[datetime] = Field(default=None)
    _host_url: Optional[str] = PrivateAttr(default=None)

    def __init__(self, _host_url: Optional[str] = None, **kwargs: Any) -> None:
        """Initialize a Run object."""
        super().__init__(**kwargs)
        self._host_url = _host_url

    @property
    def url(self) -> Optional[str]:
        """URL of this run within the app."""
        if self._host_url:
            return f"{self._host_url}/datasets/{self.id}"
        return None


class RunTypeEnum(str, Enum):
    """(Deprecated) Enum for run types. Use string directly."""

    tool = "tool"
    chain = "chain"
    llm = "llm"
    retriever = "retriever"
    embedding = "embedding"
    prompt = "prompt"
    parser = "parser"


class RunBase(BaseModel):
    """
    Base Run schema.
    Contains the fundamental fields to define a run in a system.
    """

    id: UUID
    """Unique identifier for the run."""

    name: str
    """Human-readable name for the run."""

    start_time: datetime
    """Start time of the run."""

    run_type: str
    """The type of run, such as tool, chain, llm, retriever,
    embedding, prompt, parser."""

    end_time: Optional[datetime] = None
    """End time of the run, if applicable."""

    extra: Optional[dict] = None
    """Additional metadata or settings related to the run."""

    error: Optional[str] = None
    """Error message, if the run encountered any issues."""

    serialized: Optional[dict] = None
    """Serialized object that executed the run for potential reuse."""

    events: Optional[List[Dict]] = None
    """List of events associated with the run, like
    start and end events."""

    inputs: dict
    """Inputs used for the run."""

    outputs: Optional[dict] = None
    """Outputs generated by the run, if any."""

    reference_example_id: Optional[UUID] = None
    """Reference to an example that this run may be based on."""

    parent_run_id: Optional[UUID] = None
    """Identifier for a parent run, if this run is a sub-run."""

    tags: Optional[List[str]] = None
    """Tags for categorizing or annotating the run."""


class Run(RunBase):
    """Run schema when loading from the DB."""

    execution_order: int
    """The execution order of the run within a run trace."""
    session_id: Optional[UUID] = None
    """The project ID this run belongs to."""
    child_run_ids: Optional[List[UUID]] = None
    """The child run IDs of this run."""
    child_runs: Optional[List[Run]] = None
    """The child runs of this run, if instructed to load using the client
    These are not populated by default, as it is a heavier query to make."""
    feedback_stats: Optional[Dict[str, Any]] = None
    """Feedback stats for this run."""
    app_path: Optional[str] = None
    """Relative URL path of this run within the app."""
    manifest_id: Optional[UUID] = None
    """Unique ID of the serialized object for this run."""
    status: Optional[str] = None
    """Status of the run (e.g., 'success')."""
    prompt_tokens: Optional[int] = None
    """Number of tokens used for the prompt."""
    completion_tokens: Optional[int] = None
    """Number of tokens generated as output."""
    total_tokens: Optional[int] = None
    """Total tokens for prompt and completion."""
    first_token_time: Optional[datetime] = None
    """Time the first token was processed."""
    parent_run_ids: Optional[List[UUID]] = None
    """List of parent run IDs."""
    trace_id: Optional[UUID] = None
    """Unique ID assigned to every run within this nested trace."""
    dotted_order: Optional[str] = None
    """Dotted order for the run.

    This is a string composed of {time}{run-uuid}.* so that a trace can be
    sorted in the order it was executed.

    Example:
    - Parent: 20230914T223155647Z1b64098b-4ab7-43f6-afee-992304f198d8
    - Children:
        - 20230914T223155647Z1b64098b-4ab7-43f6-afee-992304f198d8.20230914T223155649Z809ed3a2-0172-4f4d-8a02-a64e9b7a0f8a
        - 20230915T223155647Z1b64098b-4ab7-43f6-afee-992304f198d8.20230914T223155650Zc8d9f4c5-6c5a-4b2d-9b1c-3d9d7a7c5c7c
    """  # noqa: E501
    _host_url: Optional[str] = PrivateAttr(default=None)

    def __init__(self, _host_url: Optional[str] = None, **kwargs: Any) -> None:
        """Initialize a Run object."""
        super().__init__(**kwargs)
        self._host_url = _host_url

    @property
    def url(self) -> Optional[str]:
        """URL of this run within the app."""
        if self._host_url and self.app_path:
            return f"{self._host_url}{self.app_path}"
        return None


class FeedbackSourceBase(BaseModel):
    type: str
    metadata: Optional[Dict[str, Any]] = Field(default_factory=dict)


class APIFeedbackSource(FeedbackSourceBase):
    """API feedback source."""

    type: Literal["api"] = "api"


class ModelFeedbackSource(FeedbackSourceBase):
    """Model feedback source."""

    type: Literal["model"] = "model"


class FeedbackSourceType(Enum):
    """Feedback source type."""

    API = "api"
    """General feedback submitted from the API."""
    MODEL = "model"
    """Model-assisted feedback."""


class FeedbackBase(BaseModel):
    """Feedback schema."""

    id: UUID
    """The unique ID of the feedback."""
    created_at: Optional[datetime] = None
    """The time the feedback was created."""
    modified_at: Optional[datetime] = None
    """The time the feedback was last modified."""
    run_id: UUID
    """The associated run ID this feedback is logged for."""
    key: str
    """The metric name, tag, or aspect to provide feedback on."""
    score: SCORE_TYPE = None
    """Value or score to assign the run."""
    value: VALUE_TYPE = None
    """The display value, tag or other value for the feedback if not a metric."""
    comment: Optional[str] = None
    """Comment or explanation for the feedback."""
    correction: Union[str, dict, None] = None
    """Correction for the run."""
    feedback_source: Optional[FeedbackSourceBase] = None
    """The source of the feedback."""

    class Config:
        frozen = True


class FeedbackCreate(FeedbackBase):
    """Schema used for creating feedback."""

    feedback_source: FeedbackSourceBase
    """The source of the feedback."""


class Feedback(FeedbackBase):
    """Schema for getting feedback."""

    id: UUID
    created_at: datetime
    """The time the feedback was created."""
    modified_at: datetime
    """The time the feedback was last modified."""
    feedback_source: Optional[FeedbackSourceBase] = None
    """The source of the feedback. In this case"""


class TracerSession(BaseModel):
    """TracerSession schema for the API.

    Sessions are also referred to as "Projects" in the UI.
    """

    id: UUID
    """The ID of the project."""
    start_time: datetime = Field(default_factory=datetime.utcnow)
    """The time the project was created."""
    name: Optional[str] = None
    """The name of the session."""
    extra: Optional[Dict[str, Any]] = None
    """Extra metadata for the project."""
    tenant_id: UUID
    """The tenant ID this project belongs to."""

    _host_url: Optional[str] = PrivateAttr(default=None)

    def __init__(self, _host_url: Optional[str] = None, **kwargs: Any) -> None:
        """Initialize a Run object."""
        super().__init__(**kwargs)
        self._host_url = _host_url

    @property
    def url(self) -> Optional[str]:
        """URL of this run within the app."""
        if self._host_url:
            return f"{self._host_url}/o/{self.tenant_id}/projects/p/{self.id}"
        return None


class TracerSessionResult(TracerSession):
    """TracerSession schema returned when reading a project
    by ID. Sessions are also referred to as "Projects" in the UI."""

    run_count: Optional[int]
    """The number of runs in the project."""
    latency_p50: Optional[timedelta]
    """The median (50th percentile) latency for the project."""
    latency_p99: Optional[timedelta]
    """The 99th percentile latency for the project."""
    total_tokens: Optional[int]
    """The total number of tokens consumed in the project."""
    prompt_tokens: Optional[int]
    """The total number of prompt tokens consumed in the project."""
    completion_tokens: Optional[int]
    """The total number of completion tokens consumed in the project."""
    last_run_start_time: Optional[datetime]
    """The start time of the last run in the project."""
    feedback_stats: Optional[Dict[str, Any]]
    """Feedback stats for the project."""
    reference_dataset_ids: Optional[List[UUID]]
    """The reference dataset IDs this project's runs were generated on."""
    run_facets: Optional[List[Dict[str, Any]]]
    """Facets for the runs in the project."""


@runtime_checkable
class BaseMessageLike(Protocol):
    """
    A protocol representing objects similar to BaseMessage.
    """

    content: str
    additional_kwargs: Dict

    @property
    def type(self) -> str:
        """Type of the Message, used for serialization."""




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time     : 
# @Author   : wgh
from FourthDimension.interface.generate_answer import generate_answers
from FourthDimension.interface.upload import upload_entrance
from FourthDimension.interface.query import query_entrance
from FourthDimension.interface.clean import clean_entrance
from FourthDimension.interface.parse import parse_data


def upload(doc_path):
    """
    文档上传接口
    数据格式List[Document]
    :param doc_path: 文档路径
    :return:
    """
    all_contexts = parse_data(doc_path)
    upload_entrance(all_contexts)


def query(question):
    """
    检索增强生成(问答)接口
    :param question: 问题
    :return:
    """
    top_k_contexts = query_entrance(question)
    answer = generate_answers(question, top_k_contexts)
    return answer


def clean():
    """
    清除所有数据
    """
    clean_entrance()



import json
from typing import Union
import faiss
import jieba
import numpy as np
from FourthDimension.config.config import embedding_model


def load_json(path):
    with open(path, "r", encoding="utf-8") as f:
        data = json.load(f)
    f.close()
    return data


def write_json(path, data):
    with open(path, "w", encoding="utf-8") as f:
        json_data = json.dumps(data, ensure_ascii=False, indent=4)
        f.write(json_data)
    f.close()


def chinese_segment(text):
    sentences = []
    seg_list = jieba.cut(text, cut_all=False)
    sentence = []
    for word in seg_list:
        sentence.append(word)
        if word in ['。', '？', '！']:
            sentences.append("".join(sentence))
            sentence = []
    if sentence:
        sentences.append("".join(sentence))
    return sentences

def index_search(query_embed, context_embed):
    contexts_embeddings = np.array([context_embed]).astype("float32")
    index = faiss.IndexFlatL2(contexts_embeddings.shape[1])  # 创建Faiss索引
    index.add(contexts_embeddings)
    query_embedding = np.array([query_embed]).astype("float32")
    D, I = index.search(query_embedding, 1)
    return D


def get_simi_score(question, context):
    query_embed = embedding_model.embed_query(question)
    contexts_embed = embedding_model.embed_query(context)
    D = index_search(query_embed=query_embed, context_embed=contexts_embed)
    return D



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time     : 
# @Author   : wgh


"""
文件说明：
"""
from FourthDimension.utils.file_parse import get_all_docx_contexts


def parse_data(doc_path):
    """
    数据解析
    :param doc_path: 文档路径
    :return:
    """
    print('开始文档解析...')
    all_contexts = get_all_docx_contexts(doc_path)
    return all_contexts



import json
from FourthDimension.doc.pydantic import Field
from FourthDimension.doc.document import Document


class DocumentSerializable(Document):
    """Class for storing a piece of text and associated metadata."""

    page_content: str
    metadata: dict = Field(default_factory=dict)

    @property
    def lc_serializable(self) -> bool:
        return True

    def __repr__(self):
        return f"Document(page_content='{self.page_content[:50]}...', metadata={self.metadata})"

    def __str__(self):
        return self.__repr__()

    def to_json(self) -> str:
        """Convert the Document object to a JSON string."""
        return json.dumps(
            {
                "page_content": self.page_content,
                "metadata": self.metadata,
            }
        )

    @classmethod
    def from_json(cls, json_str: str):
        """Create a Document object from a JSON string."""
        data = json.loads(json_str)
        return cls(page_content=data["page_content"], metadata=data["metadata"])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time     : 
# @Author   : wgh


"""
文件说明：
"""
import json
import os

from FourthDimension.parser.docx_parser import parse_docx


def get_file_paths_and_names(folder_path):
    if '.doc' in folder_path or '.docx' in folder_path or '.json' in folder_path:
        return [(folder_path, os.path.basename(folder_path))]
    else:
        file_paths_and_names = []
        for root, dirs, files in os.walk(folder_path):
            for file_name in files:
                file_path = os.path.join(root, file_name)
                file_paths_and_names.append((file_path, file_name))
    return file_paths_and_names


def get_all_docx_contexts(doc_path):
    all_contexts = []
    file_paths_and_names = get_file_paths_and_names(doc_path)
    for i, d in enumerate(file_paths_and_names):
        file_path = d[0]
        file_name = d[1]
        document = parse_docx(file_path, file_name)
        all_contexts.extend(document)
    return all_contexts
